@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # vim: ts=4
 ###
 #
@@ -15,35 +16,45 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 #
 ###
 
-
-import gtk,gobject
-
+import gobject
+import gtk
 import stock
-import config
-import const
+from config import config
+import utils
 
 from dbus_manager import ListenDBus
 from hal import hal
 
 from player import Player
 
-from widget.player_ui import PlayerUI
 from widget.tray import TrayIcon
 from widget.source_ui import SourceUI
-from widget.dialog import *
-from widget.preference import PreferenceDialog,SourceDialog,ViewDialog
-from library import library_wrapper
-from audioscrobbler_manager import AudioScrobblerManager
-import mmkeys_wrap
+from widget.dialog import WindowNewPodcast, ListenAbout, WindowLoadPlaylist, WinFile, WinURI
+    
+from widget.preference import PreferenceDialog
+from widget.plugins_preferences import PluginsPreferenceDialog
+from library import ListenDB
 
 from widget.notify import Notify
 from widget.osd import Osd
+from widget.misc import ScrolledWindow
+from widget.playlist import PlaylistUI
+from widget.menubar import ListenMenuBar
 
-class Listen(gtk.Window):
+from xdg_support import get_xdg_pixmap_file
+from vfs import FileMonitor, async_parse_uris
+from helper import Dispatcher
+from mmkeys_wrap import MMKeys
+from logger import Logger
+
+#TODO: Remove that
+from source.webradio import WindowNewRadio
+
+class Listen(gtk.Window,Logger):
     __obscured = False
     __gsignals__ = {
         "ready" : (gobject.SIGNAL_RUN_LAST,
@@ -53,9 +64,8 @@ class Listen(gtk.Window):
         
         #Start Audioscrobbler Session
         
-        AudioScrobblerManager()
         gtk.Window.__init__(self,gtk.WINDOW_TOPLEVEL)
-        gtk.window_set_default_icon_from_file(const.PIXMAP_DIR+"listen.png")
+        gtk.window_set_default_icon_from_file( get_xdg_pixmap_file("listen.png") )
         
         self.set_title("Listen")
         
@@ -66,7 +76,6 @@ class Listen(gtk.Window):
 
         try: self.resize(int(config.get("window","width")),int(config.get("window","height")))
         except:pass
-        #self.resize(800,600)
         
                 
         window_state = config.get("window","state")
@@ -74,55 +83,58 @@ class Listen(gtk.Window):
             self.maximize()
         if window_state == "normal":
             self.unmaximize()
-        
-
 
         self.global_box = gtk.HPaned()
-        #self.global_box.set_spacing(18)
         self.global_box.set_border_width(6)
         self.global_box.compute_position(-1,300,True)
         
-        player = Player()
-        
-        self.source_ui = SourceUI(player)
-        self.player_ui = PlayerUI(player)
-        
-        #self.global_box.pack_start(self.player_ui,False,False)
-        #self.global_box.pack_start(self.source_ui,True,True)
-        
-        self.global_box.pack1(self.player_ui,False,False)
-        self.global_box.pack2(self.source_ui,True,True)
+        self.menubar = ListenMenuBar()
+
+        self.source_ui = SourceUI()
+        self.playlist_ui = PlaylistUI(self.menubar)
+         
+        # Set player source to a static playlist for now
+        Player.set_source(self.playlist_ui.playlist)
+
+        if config.get("window","right_layout") == "true":
+            self.global_box.pack1(self.source_ui,False,False)
+            self.global_box.pack2(self.playlist_ui,True,True)
+        else:
+            self.global_box.pack1(self.playlist_ui,False,False)
+            self.global_box.pack2(self.source_ui,True,True)
+        self.global_box.child_set_property(self.global_box.get_child1(),"resize",True)
+        self.global_box.child_set_property(self.global_box.get_child2(),"resize",True)
 
         self.global_box.set_position(int(config.get("window","pos_global_pane")))
         
-        #Attach accel from playlist
-        self.add_accel_group(self.player_ui.playlist.ui.get_accel_group())
         
-        vbox = gtk.VBox()
+        #Attach accel from playlist
+        self.add_accel_group(self.playlist_ui.playlist.ui.get_accel_group())
+
+        vbox = gtk.VBox(spacing=0)
         vbox.pack_start(self.get_main_menu(),False,False)
         vbox.pack_start(self.global_box,True,True)
         self.add(vbox)
 
+        Dispatcher.connect("quit",self.force_quit)
         self.connect("delete-event",self.quit)
         self.connect("configure-event",self.on_configure_event)
         
-        player.connect("instant-new-song",self.update_title)
-        helper.connect("changed",self.update_title)
+        Player.connect("instant-new-song",self.update_title)
+        ListenDB.connect("simple-changed",self.update_title)
         
-        player.connect("played",self.swap_menu_play,player,_("Pause"),gtk.STOCK_MEDIA_PAUSE,"listen_tray_play.png")
-        player.connect("paused",self.swap_menu_play,player,("Play"),gtk.STOCK_MEDIA_PLAY,"listen_tray_pause.png")
+        Player.connect("played",self.swap_menu_play,Player,_("Pause"),gtk.STOCK_MEDIA_PAUSE,"listen_tray_play.png")
+        Player.connect("paused",self.swap_menu_play,Player,("Play"),gtk.STOCK_MEDIA_PLAY,"listen_tray_pause.png")
+        Player.connect("stopped",self.swap_menu_play,Player,("Play"),gtk.STOCK_MEDIA_PLAY,"listen_tray.png")
         
-        self.tray = TrayIcon(self,player)
-        self.tray.connect("control-action",self.player_ui.control,player)
-        self.tray.connect("control-action",self.tray_control)
+        self.tray = TrayIcon(self)
 
-        self.osd = Osd(player,self.tray)
-        self.notify = Notify(player,self.tray)
-        self.dbus_service = ListenDBus(self,player)
+        self.osd = Osd(self.tray)
+        self.notify = Notify(self.tray)
+        self.dbus_service = ListenDBus(self)
+
+        self.mmkeys = MMKeys()
 
-        helper.connect("control-action",self.player_ui.control,player)
-        helper.connect("control-action",self.tray_control)
-        
         self.pos = None
         if config.get("window","view")=="3":
             config.set("window","view","2")
@@ -136,20 +148,17 @@ class Listen(gtk.Window):
         except:pass
         
         if config.get("setting","start_trayicon")!="true" or config.get("setting","use_trayicon")!="true":
-            gobject.idle_add(self.ready)
-            gobject.timeout_add(1000, hal.load)
+            gobject.idle_add(self.ready,True)
+        else:
+            gobject.idle_add(self.ready,False)
         if config.get("setting","use_trayicon")=="true" and self.tray.tray!=None:
             self.tray.tray.show_all()
             
-    def ready(self):
-        self.show_all()
+    def ready(self,show):
+        if show: self.show_all()
+        self.playlist_ui.playlist.load()
         self.emit("ready")
         
-        
-    def tray_control(self, w, action):
-        if action=="quit":
-            self.force_quit()
-        
     def quit(self,*param):
         self.tray.cacher()
         if config.get("setting","fermer_trayicon")=="false" or self.tray.tray==None:
@@ -157,40 +166,47 @@ class Listen(gtk.Window):
         return True
     
     def force_quit(self,*param):
+        Player.save_state()
+        if not Player.is_paused(): Player.pause()
         self.tray.cacher()
-        self.tray.tray_image.set_from_file(const.PIXMAP_DIR+"/listen_tray_save.png")
-        gobject.idle_add(self.__idle_quit)
+        self.tray.tray_image.set_from_file( get_xdg_pixmap_file("listen_tray_save.png") )
+        # wait fadeout done
+        gobject.timeout_add(500,self.__idle_quit)
             
     def __idle_quit(self,*param):
+        print "Exiting..."
+        Player.stop()
+        self.notify.close_notification()
         self.tray.cacher()
         config.set("window","pos_global_pane","%d"%self.global_box.get_position())
-        
         self.source_ui.save()
-        self.player_ui.playlist.save()
-        from library import library_wrapper
-        library_wrapper.save()
-        
-        config.write(const.CONFIG_FILE)
+        self.playlist_ui.playlist.save(close=True)
+        ListenDB.save()
+        FileMonitor.close()
+        config.write()
         gtk.main_quit()
         print "Exit successful"
-        
-        
-    def update_title(self,player_or_helper,song):
-        #Don't song because she can be provide by 2 different source
-        song =  self.player_ui.player.song
-        if not song: return
-        title = ""
-        sep = ""
-        if song.get_str("title"):
-            title += sep+song.get_str("title")+" "
-            sep=" - "
-        if song.get_str("artist"):
-            title += sep+song.get_str("artist")+" "
-            sep=" - "
-        if song.get_str("album"):
-            title += song.get_str("album")
-        self.set_title("Listen  "+"\xe2\x99\xaa "+title+" "+"\xe2\x99\xaa ")
-        
+     
+    def update_title(self,*args):
+        song = Player.song
+        if config.getboolean("setting", "update_title") and song: 
+            title = ""
+            sep = ""
+            if song.get_str("title"):
+                title += sep+song.get_str("title")+" "
+                sep=" - "
+            if song.get_str("artist"):
+                title += sep+song.get_str("artist")+" "
+                sep=" - "
+            if song.get_str("album"):
+                title += song.get_str("album")
+            title = title.replace("\n",",  ")
+            title = "Listen  "+"\xe2\x99\xaa "+title+" "+"\xe2\x99\xaa "
+        else:
+            title = "Listen"
+    
+        self.set_title(title)
+
     def on_configure_event(self,widget=None,event=None):
         #windows property
         if widget.get_property("visible") and config.get("window","view")!="3":
@@ -205,9 +221,10 @@ class Listen(gtk.Window):
 
     def change_view(self,num_view):
         self.source_ui.show_all()    
-        self.player_ui.box_playlist.show_all()  
         self.source_ui.set_no_show_all(True)  
-        self.player_ui.box_playlist.set_no_show_all(True)
+        self.playlist_ui.box_playlist.show_all()  
+        self.playlist_ui.box_playlist.set_no_show_all(True)
+
         if self.pos:
             self.set_keep_above(False)
             self.unfullscreen()
@@ -218,11 +235,10 @@ class Listen(gtk.Window):
             self.pos = self.get_position()
             
             self.source_ui.show()
-            self.player_ui.box_playlist.show()
+            self.playlist_ui.box_playlist.show()
             self.set_resizable(True)
             self.resize(int(config.get("window","width")),int(config.get("window","height")))
             self.global_box.set_position(int(config.get("window","pos_global_pane")))
-            monitor = gtk.gdk.Screen.get_monitor_geometry(gtk.gdk.screen_get_default(), 0)
             
             
             self.set_keep_above(True)
@@ -231,7 +247,7 @@ class Listen(gtk.Window):
             
         elif num_view == 2 :#FULL VIEW  
             self.source_ui.show()
-            self.player_ui.box_playlist.show()
+            self.playlist_ui.box_playlist.show()
             self.set_resizable(True)
             self.resize(int(config.get("window","width")),int(config.get("window","height")))
             self.global_box.set_position(int(config.get("window","pos_global_pane")))
@@ -239,31 +255,43 @@ class Listen(gtk.Window):
             
         elif num_view == 1 :#NORMAL VIEW
             self.source_ui.hide()
-            self.player_ui.box_playlist.show()
+            self.playlist_ui.box_playlist.show()
             self.set_resizable(True)
             self.resize(int(config.get("window","pos_global_pane")),int(config.get("window","height")))
             
         elif num_view == 0:#PLAYER  VIEW
             self.source_ui.hide()
-            self.player_ui.box_playlist.hide()
+            self.playlist_ui.box_playlist.hide()
             self.resize(int(config.get("window","pos_global_pane")),int(config.get("window","height")))
             self.set_resizable(False)
-            self.set_size_request(300,-1)
+            #self.set_size_request(300,-1)
 
         config.set("window","view","%d"%num_view)
         
+    def open_uri_win(self,w):
+        uri = WinURI().run()
+        if uri:
+            ListenDB.get_or_create_song({"uri":uri},"unknown")
+            self.playlist_ui.playlist.play_uris([uri])
+
+    def open_file2_win(self,w):
+        uri = WinFile().run()
+        if uri:
+           self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+           async_parse_uris( [uri], True, True,
+                    self.playlist_ui.playlist.play_uris, pos = None, sort = True)
+
     def open_file_win(self,w):
-        l = library_wrapper.get_library("local")
-        l.import_file()
+        from source.local import ImportFileJob
+        ImportFileJob()
         
     def open_folder_win(self,w):
-        l = library_wrapper.get_library("local")
-        l.import_folder()
+        from source.local import ImportFolderJob
+        ImportFolderJob()
         
     def reload_local_db(self,s):
-        l = library_wrapper.get_library("local")
-        l.reload_db()
-        
+        from source.local import ReloadDB
+        ReloadDB()
         
     def open_podcast_win(self,w):
         WindowNewPodcast()
@@ -276,60 +304,48 @@ class Listen(gtk.Window):
     
     def open_pref_dialog(self,w):
         PreferenceDialog(self,self.osd)
-    
-    def control(self,action):
-        name = action.get_name()
-        self.player_ui.control(action,name)
-        pass
-              
-    def new_playlist(self,s):
-        from library.local import LocalPlaylist
-        l = library_wrapper.get_library("local")
-        pl = LocalPlaylist(l,_("New Playlist"))
-        l.add_playlist(pl)
-        gobject.idle_add(helper.edit_playlist,pl)
         
+    def open_plugins_dialog(self,w):
+        PluginsPreferenceDialog(self)
+    
+    def player_control(self,action):
+        name = action.get_name()
+        getattr(Player, name)()
+              
+    def menu_control(self,action):
+        name = action.get_name()
+        getattr(self.playlist_ui.playlist, name)()
+
+    def new_playlist(self,s):
+        ListenDB.create_playlist("local","New Playlist")
         
     def new_playlist_smart(self,s):
-        from library import PL_SMART
-        from library.local import LocalPlaylist
-        from widget.smart_playlist import SmartPlaylistWindow
-        l = library_wrapper.get_library("local")
-        pl = LocalPlaylist(l,_("New Automatic Playlist"),PL_SMART)   
-        win = SmartPlaylistWindow(pl,True)
-        pl = win.run()
-        if pl:
-            l.add_playlist(pl)
-            gobject.idle_add(helper.edit_playlist,pl)
-    
+        ListenDB.create_autoplaylist("local","local","New Automatic Playlist")
         
     def download_cover(self,w):
-        library_wrapper.get_library("local").download_all_cover()
+        from source.local import CoverDownloadJob
+        CoverDownloadJob()
     
     def load_playlist(self,s):
         WindowLoadPlaylist().run()
     
     def save_playlist(self,s):
-        self.player_ui.playlist.save_to_library()
+        self.playlist_ui.playlist.save_to_library()
     
-    
-    def edit_source(self,s):
-        SourceDialog(self)
-        
-    def edit_column(self,s):
-        ViewDialog(self)
-        
     def swap_menu_play(self,w,player,name,stock,image):
-        action = self.ui.get_action("/MenuBar/Control/play")   
+        action = self.ui.get_action("/MenuBar/Control/playpause")   
         action.set_property("stock-id",stock)
         action.set_property("label",name)
-        self.tray.tray_image.set_from_file(const.PIXMAP_DIR+"/"+image)
+        self.tray.tray_image.set_from_file( get_xdg_pixmap_file(image) )
         
     def get_main_menu(self):
         menu = """
         <ui>
           <menubar name="MenuBar">
             <menu action="Music">
+              <menuitem action="OpenFile"/>
+              <menuitem action="OpenUri"/>
+              <separator />
               <menuitem action="AddFile"/>
               <menuitem action="AddPath"/>
               <separator />
@@ -347,6 +363,7 @@ class Listen(gtk.Window):
               <menuitem action="AddPodcast"/>
               <separator />
               <menuitem action="Config"/>
+              <menuitem action="Plugins"/>
               <menuitem action="OfflineMode"/>
               <separator />
               <menuitem action="Quit"/>
@@ -362,12 +379,11 @@ class Listen(gtk.Window):
                   <menuitem action="browserview1"/>
                   <menuitem action="browserview2"/>
               </menu>
-              <separator />
-              <menuitem action="EditSource"/>
-              <menuitem action="EditColumn"/>
             </menu>
             <menu action="Control">
-              <menuitem action="play"/>
+              <menuitem action="playpause"/>
+              <menuitem action="rewind"/>
+              <menuitem action="forward"/>
               <separator />
               <menuitem action="previous"/>
               <menuitem action="next"/>
@@ -383,6 +399,8 @@ class Listen(gtk.Window):
         </ui>
         """
         
+        #<menuitem action="stop"/>
+
         ag = gtk.ActionGroup('Listen')
         
         ag.add_actions([('Music', None, _('_Music')),
@@ -393,28 +411,31 @@ class Listen(gtk.Window):
                       ('Control', None, _('_Control')),
                       ('Playlist', None, _('_Playlist')),
                       ('MediaSource', None, _('Middle pane')),
+                      ('OpenFile', stock.OPEN_FILE, None, "","", self.open_file2_win),
+                      ('OpenUri', stock.OPEN_URI, None, "","", self.open_uri_win),
                       ('AddFile', stock.IMPORT_FILE,None,None,"", self.open_file_win),
-                      ('AddPath', stock.IMPORT_FOLDER, None, "<control>o","", self.open_folder_win),
+                      ('AddPath', stock.IMPORT_FOLDER, None, "<Control>o","", self.open_folder_win),
                       ('ReloadDB',gtk.STOCK_REFRESH,_("Reload Library"),"","",self.reload_local_db),
                         
-                      ('AddRadio', gtk.STOCK_NEW, _('New Internet Radio Station'), "<control>i","", self.open_radio_win),
-                      ('AddPodcast', gtk.STOCK_NEW, _('_New Podcast feed'), "<control>p","", self.open_podcast_win),
+                      ('AddRadio', gtk.STOCK_NEW, _('New Internet Radio Station'), "<Control>i","", self.open_radio_win),
+                      ('AddPodcast', gtk.STOCK_NEW, _('_New Podcast feed'), "<Control>p","", self.open_podcast_win),
                       
-                      ('NewPlaylist', stock.SRC_PLAYLIST, _('_New Playlist'), "<control>n","", self.new_playlist),
+                      ('NewPlaylist', stock.SRC_PLAYLIST, _('_New Playlist'), "<Control>n","", self.new_playlist),
                       ('NewPlaylistSmart', stock.SRC_PLAYLIST_SMART, _('New _Automatic Playlist...'), None,"", self.new_playlist_smart),
                       ('LoadPlaylist', None, _('Load from file...'), None,"", self.load_playlist),
-                      ('SavePlaylist', gtk.STOCK_SAVE , _('Save Current Playlist'), "<control>s","", self.save_playlist),
+                      ('SavePlaylist', gtk.STOCK_SAVE , _('Save Current Playlist'), "<Control>s","", self.save_playlist),
 
                       ('About', gtk.STOCK_ABOUT, _('_About'), None,"", self.open_about_dialog),
 
                       ('Config', gtk.STOCK_PREFERENCES, _('P_references'), None,"", self.open_pref_dialog),
-                      ('Quit', gtk.STOCK_QUIT, _('_Quit'), "<control>q","", self.force_quit),
-                      ('play', gtk.STOCK_MEDIA_PLAY, _('_Play'), "<control>space","", self.control),
-                      ('previous', gtk.STOCK_MEDIA_PREVIOUS, _('_Previous'), "<alt>Left","", self.control),
-                      ('next', gtk.STOCK_MEDIA_NEXT, _('_Next'), "<alt>Right","", self.control),
-                      ('shuffle', stock.SHUFFLE, _('_Shuffle'), "<control>u","", self.control),
-                      ('EditSource',None,_('Source...'),None,"",self.edit_source),
-                      ('EditColumn',None,_('Column...'),None,"",self.edit_column),
+                      ('Plugins', gtk.STOCK_PREFERENCES, _('Plugins'), None,"", self.open_plugins_dialog),
+                      ('Quit', gtk.STOCK_QUIT, _('_Quit'), "<Control>q","", self.force_quit),
+                      ('playpause', gtk.STOCK_MEDIA_PLAY, _('_Play'), "<Control>space","", self.player_control),
+                      #('stop', gtk.STOCK_MEDIA_STOP, _('_Stop'), None,"", self.player_control),
+                      ('previous', gtk.STOCK_MEDIA_PREVIOUS, _('_Previous'), "<Alt>Left","", self.player_control),
+                      ('next', gtk.STOCK_MEDIA_NEXT, _('_Next'), "<Alt>Right","", self.player_control),
+                      ('rewind', gtk.STOCK_MEDIA_REWIND, _('_Rewind'), "<Control>Left","", self.player_control),
+                      ('forward', gtk.STOCK_MEDIA_FORWARD, _('_Forward'), "<Control>Right","", self.player_control),
                       ("DownloadCover",None,_("Download missing cover"),None,"",self.download_cover)
                       
                      ])
@@ -437,23 +458,27 @@ class Listen(gtk.Window):
                               ('Full', None, _('Full Display'), "<control>f",None,2),
                               ('Party', None, _('Party mode'), "<control>h",None,3)],int(config.get("window","view")), on_change_view)
                                                      
-                           
         ag.add_toggle_actions([
            ('repeat', None, _('_Repeat'), "<control>r",None,None,config.getboolean("setting","repeat_mode")),
+           ('shuffle', stock.SHUFFLE, _('_Shuffle'), "<control>u",None,None,config.getboolean("setting","shuffle_mode")),
            ('OfflineMode',None,_('Offline Mode'),None,None,None,config.getboolean("setting","offline"))
                       ])
-        b1 = ag.get_action("repeat")
-        b1.connect_proxy(self.player_ui.playlist_control.btn_repeat)
-        if config.getboolean("setting","repeat_mode"): b1.activate()
+        b = ag.get_action("repeat")
+        b.connect_proxy(self.playlist_ui.playlist_control.btn_repeat)
+        if config.getboolean("setting","repeat_mode"): b.activate()
 
-        b2 = ag.get_action("OfflineMode")
+        b = ag.get_action("shuffle")
+        b.connect_proxy(self.playlist_ui.playlist_control.btn_shuffle)
+        if config.getboolean("setting","shuffle_mode"): b.activate()
+
+        b = ag.get_action("OfflineMode")
         def toggle_offline(w): 
             if w.get_active(): config.set("setting","offline","true")
             else: config.set("setting","offline","false") 
-        b2.connect("toggled",toggle_offline)
-        if config.getboolean("setting","offline"): b2.activate()
+        b.connect("toggled",toggle_offline)
+        if config.getboolean("setting","offline"): b.activate()
 
-        def change(action,*pazoram):
+        def change(action,*param):
             name = action.get_property("name")
             section = name[:name.index("_")]
             option = name[name.index("_")+1:]
@@ -470,4 +495,3 @@ class Listen(gtk.Window):
         
         return self.ui.get_widget("/MenuBar")
 
-
